<?php
/**
* DT Structure Class
* Abstract Data Transaction Structure Class
* @package		phpOpenFW
* @subpackage 	Database_Tools
* @author 		Christian J. Clark
* @copyright	Copyright (c) Christian J. Clark
* @license		http://www.gnu.org/licenses/gpl-2.0.txt
* @access		private
* @version 		Started: 2-1-2007 updated: 3-24-2010
*/

//***************************************************************
/**
 * dt_structure Class
 * @package		phpOpenFW
 * @subpackage	Database_Tools
 * @access		private
 */
//***************************************************************
abstract class dt_structure {

	//*************************************************************************	
	// Class variables
	//*************************************************************************
	/**
	* @var string data source type (mysql, mysqli, pgsql, ldap, oracle, sqlite, mssql)
	**/
	protected $data_type;
	
	/**
	* @var string Server address to use when connecting to the data source
	**/
	protected $server;
	
	/**
	* @var string Server port to use when connecting to the data source
	**/
	protected $port;
	
	/**
	* @var string Database or directory to use when connecting to the data source
	**/
	protected $source;

	/**
	* @var string Database instance to use when connecting to the data source
	**/
	protected $instance;
	
	/**
	* @var string User to use when connecting to the data source
	**/
	protected $user;
	
	/**
	* @var string Password to use when connecting to the data source
	**/
	protected $pass;

	/**
	* @var string Database specific connection string
	**/
	protected $conn_str;
	
	/**
	* @var string Data transaction type
	**/
	protected $trans_type;
	
	/**
	* @var resource Connection handle of the current data source connection
	**/
	protected $handle;
	
	/**
	* @var resource Resource ID of the data source record set
	**/
	protected $rsrc_id;
	
	/**
	* @var array The record set
	**/
	protected $result;
	
	/**
	* @var integer Number of rows in the record set returned
	**/
	protected $num_rows;

	/**
	* @var integer Number of fields in the record set returned
	**/
	protected $num_fields;
	
	/**
	* @var integer ID of the last inserted row for the current connection
	**/
	protected $last_id;
	
	/**
	* @var bool User is bound to data source for this transaction (true or false) 
	**/
	protected $bound;

	/**
	* @var bool Connection Open (true or false) 
	**/
	protected $conn_open;

	/**
	* @var string Display Data Type 
	**/
	protected $disp_dt;

	/**
	* Constructor function
	*
	* Initializes data transaction, setting all necessary variables from specified data source
	*
	* @param string The name of data source to use as specified in config.inc.php
	**/
	//*************************************************************************
	// Constructor function
	//*************************************************************************
	public function __construct($data_src, $tmp_trans_type = 'qry')
	{
        if (array_key_exists($data_src, $_SESSION)) {
			// Set Data Source Variables
			$this->data_type = (isset($_SESSION[$data_src]['type'])) ? ($_SESSION[$data_src]['type']) : (false);
			$this->server = (isset($_SESSION[$data_src]['server'])) ? ($_SESSION[$data_src]['server']) : (false);
			$this->port = (isset($_SESSION[$data_src]['port'])) ? ($_SESSION[$data_src]['port']) : (false);
			$this->instance = (isset($_SESSION[$data_src]['instance'])) ? ($_SESSION[$data_src]['instance']) : (false);
			$this->source = (isset($_SESSION[$data_src]['source'])) ? ($_SESSION[$data_src]['source']) : (false);
			$this->user = (isset($_SESSION[$data_src]['user'])) ? ($_SESSION[$data_src]['user']) : (false);
			$this->pass = (isset($_SESSION[$data_src]['pass'])) ? ($_SESSION[$data_src]['pass']) : (false);
			$this->conn_str = (isset($_SESSION[$data_src]['conn_str'])) ? ($_SESSION[$data_src]['conn_str']) : (false);
			$this->trans_type = $tmp_trans_type;
			$this->result = array();
			$this->num_rows = 0;
			$this->num_fields = 0;
			$this->print_query = false;
			$this->bound = false;
			$this->last_id = NULL;
			$this->conn_open = false;
			$this->disp_dt = '[' . strtoupper($this->data_type) . ']';
			
			// Set Debug Level
			if (isset($_SESSION['data_debug']) && $_SESSION['data_debug']) {
				$this->print_query = true;
			}
		}
		else {
			trigger_error('Invalid data source!!');
		}
	}

    //*************************************************************************
	/**
	* Destructor Function
	**/
	//*************************************************************************
    public function __destruct()
    {
    	if ($this->is_open()) { $this->close(); }
    }

	/**
	* Return the current record set in the form of an associative array
	* @return array current record set in the form of an associate array 
	**/
	//*************************************************************************
	// Extract the record set to local variables
	//*************************************************************************
	public function assoc_result() { return $this->result; }

    /**
	* Set the current transaction type
	* @param string Query Type: "?" - query (default, all SQL queries), "+" - add (LDAP), "~" - modify (LDAP), 
	* "-" - delete (LDAP), "?1" - one level query (LDAP)
	**/
	//*************************************************************************
	// Set the Transaction type
	// "?" - query (default), "+" - add, "~" - modify, "-" - delete
	// "?1" - one level query (LDAP)
	//*************************************************************************
	public function set_trans_type($trans_type)
	{
		$this->trans_type = $trans_type;	
	}

	//*************************************************************************
	/**
	* Return the number rows in the current record set
	* @return the number rows in the current record set
	**/
	//*************************************************************************
	public function num_rows() { return $this->num_rows; }

	//*************************************************************************
	/**
	* Return the number fields in the current record set
	* @return the number fields in the current record set
	**/
	//*************************************************************************
	public function num_fields() { return $this->num_fields; }

    /**
	* Return ID of the last inserted row for the current session
	* @return integer ID of the last inserted row for the current session
	**/
	//*************************************************************************
	// Return the ID of the last inserted row
	//*************************************************************************
	public function last_insert_id() { return $this->last_id; }
	
	//*************************************************************************
	/**
	* Clear Result Function
	**/
	//*************************************************************************
    public function clear_result() { $this->result = array(); }
    
    //*************************************************************************
	/**
	* Clear Last Insert ID Function
	**/
	//*************************************************************************
    public function clear_last_insert_id() { $this->last_id = NULL; }

    //*************************************************************************
	/**
	* Return whether the current user is bound to the current data source
	* @return whether the current user is bound to the current data source
	**/
	//*************************************************************************
	public function is_bound() { return $this->bound; }

    //*************************************************************************
	/**
	* Get User Function
	**/
	//*************************************************************************
    public function get_user() { return $this->user; }
    
    //*************************************************************************
	/**
	* Get Password Function
	**/
	//*************************************************************************
    public function get_pass() { return $this->pass; }

    //*************************************************************************
	/**
	* Check if connection is open Function
	**/
	//*************************************************************************
    public function is_open() { return $this->conn_open; }

    //*************************************************************************
	/**
	* Connection Error Function
	**/
	//*************************************************************************
    protected function connection_error($error=false)
    {
    	if ($error) {
	    	trigger_error("$this->disp_dt Connection Error: " . $error);
		}
    }

    //*************************************************************************
	/**
	* Generic Error Function
	**/
	//*************************************************************************
    protected function gen_error($error=false)
    {
    	if ($error) {
	    	trigger_error("$this->disp_dt Error: " . $error);
		}
    }

	//*************************************************************************
	/**
	* Bind to a Data Source (Used for Authentication Primarily)
	* @param string userid of the user trying to bind to data source
	* @param string password of the user trying to bind to data source
	**/
	//*************************************************************************
	// Bind to a data source as a specified user
	// *** Used for Authentication
	//*************************************************************************
	public function bind($user, $pass)
	{
		$ret_val = false;

		switch ($this->data_type) {
			case 'mysql':
			case 'pgsql':
			case 'mysqli':
			case 'oracle':
			case 'sqlite':
			case 'mssql':
				if ($this->result[0][$_SESSION['auth_pass_field']] == $pass) {
					$ret_val = true;
					$this->bound = true;
				}
				break;

			case 'ldap':
				$this->open();
				$bind = @ldap_bind($this->handle, $user, $pass);
				if ($bind) {
					$ret_val = true;
					$this->bound = true;
				}
				break;
		}
		
		return $ret_val;
	}

    /**
	* Return an abbreviated form of the current record set in the form of an associative array
	* @param string field to be used as the "key" in the associative array
	* @param string field to be used as the "value" in the associative array
	* @return array abbreviated form of the current record set in the form of an associative array 
	**/
	//*************************************************************************
	// Extract an abbreviated record set to a "key" => "value" array
	//*************************************************************************
	public function key_val_result($key, $value)
	{		
   		// Initialize Result Array
		$assoc_result = array();
		
		switch ($this->data_type) {
			case 'mysql':
			case 'pgsql':
			case 'mysqli':
			case 'oracle':
			case 'sqlite':
			case 'mssql':
				foreach ($this->result as $row) {
					if (array_key_exists($key, $row) && array_key_exists($value, $row)) {
						$assoc_result[$row[$key]] = $row[$value];
					}
					else {
						trigger_error("$this->disp_dt key_val_result() :: Index \"$key\" or Index \"$value\" does not exist!!");
						return false;
					}
				}
				break;

			case 'ldap':
				for($a = 0; $a < $this->result['count']; $a++) {
					$tmp_array = $this->result[$a];
					if (array_key_exists($key, $tmp_array)) {
						$assoc_key = ($key == 'dn' || $key == 'count') ? ($tmp_array[$key]) : ($tmp_array[$key][0]);
						$assoc_value = '';
						if (array_key_exists($value, $tmp_array)) {
							$assoc_value = ($value == 'dn' || $value == 'count') ? ($tmp_array[$value]) : ($tmp_array[$value][0]);
						}
						$assoc_result[$assoc_key] = $assoc_value;
					}
					else {
						trigger_error("$this->disp_dt key_val_result() :: Index \"$key\" or Index \"$value\" does not exist!!");
						return false;
					}
				}
				break;
		}
		
		// Return Result Array		
		return $assoc_result;
	}

	//*************************************************************************
	/**
	* Return the current record set in the form of an associative array with $key as the key for each record
	* @param string The index of the field to be used as the "key" for each record
	* @return array Return the current record set in the form of an associative array with $key as the key for each record
	**/
	//*************************************************************************
	public function key_assoc_result($key)
	{
		// Initialize Result Array
		$assoc_result = array();
		
		switch ($this->data_type) {
			case 'mysql':
			case 'pgsql':
			case 'mysqli':
			case 'oracle':
			case 'sqlite':
			case 'mssql':
				foreach ($this->result as $row) {
					if (array_key_exists($key, $row)) {
						$assoc_result[$row[$key]] = $row;
					}
					else{
						trigger_error("$this->disp_dt key_assoc_result() :: Index \"$key\" does not exist!!");
						return false;
					}
				}
				break;

			case 'ldap':
				for($a = 0; $a < $this->result['count']; $a++) {
					$tmp_array = $this->result[$a];
					if (array_key_exists($key, $tmp_array)) {
						$assoc_key = ($key == 'dn' || $key == 'count') ? ($tmp_array[$key]) : ($tmp_array[$key][0]);
						$assoc_result[$assoc_key] = $tmp_array;
					}
					else{
						trigger_error("$this->disp_dt key_assoc_result() :: Index \"$key\" does not exist!!");
						return false;
					}
				}
				break;
		}
		
		// Return Result Array		
		return $assoc_result;
	}

}

?>